import { EditorViewConfig, ViewUpdate, EditorView } from '@codemirror/view';
import { Extension } from '@codemirror/state';
import { Options } from '@cmshiki/shiki';

type CMEditorOptions = EditorViewConfig & {
    /** listen view change, doc change, viewport change, other view update */
    onUpdate?: (u: ViewUpdate) => void;
};
type ShikiEditorOptions = Options & CMEditorOptions;

declare class ShikiEditor {
    private readonly options;
    view: EditorView;
    getTheme: Promise<(name?: string, view?: EditorView) => Extension>;
    private registerInternal;
    constructor(options: ShikiEditorOptions);
    /**
     * Dynamically set listening events,listen when the document changed.
     * @param {(u: ViewUpdate)}
     */
    setOnUpdate(callback: ShikiEditorOptions['onUpdate']): void;
    /**
     * Dynamically set listening events,listen when the document changed.
     * @param {(u: ViewUpdate)}
     */
    destroy(): void;
    update(options: Options): void;
    /**
     * change theme
     *
     * @param {string} name - theme style name,like `light`、`dark`、`dim`
     */
    changeTheme(name: string): Promise<void>;
    getValue(): string;
    setValue(newCode: string): void;
}

export { ShikiEditor };
