import * as shiki from 'shiki';
import { codeToHtml, CodeOptionsSingleTheme } from 'shiki';
import { Position, CodeJar } from 'codejar';

type CodeJarType = Parameters<typeof CodeJar>;
type CodeJarOptions = CodeJarType['2'];
type Options = {
    /** editor's content */
    code?: string;
    /** Use the modern JavaScript RegExp engine to implement the OnigScanner. */
    useJSEngine?: boolean;
    /** code's language*/
    lang?: Parameters<typeof codeToHtml>[1]['lang'];
} & Partial<CodeOptionsSingleTheme>;
type SimpleEditor = Awaited<ReturnType<typeof createEditor>>;
declare function createEditor(el: HTMLElement, options: CodeJarOptions & Options): Promise<{
    updateOptions(newOptions: Partial<{
        tab: string;
        indentOn: RegExp;
        moveToNewLine: RegExp;
        spellcheck: boolean;
        catchTab: boolean;
        preserveIdent: boolean;
        addClosing: boolean;
        history: boolean;
        window: typeof window;
    }>): void;
    updateCode(code: string): void;
    onUpdate(callback: (code: string) => void): void;
    toString: () => string;
    save: () => Position;
    restore: (pos: Position) => void;
    recordHistory: () => void;
    destroy(): void;
} & {
    destroy: () => void;
    update(options: CodeJarOptions & {
        code?: string;
        lang?: Options["lang"];
        theme?: Options["theme"];
    }): void;
    loadLanguage: (...langs: (shiki.SpecialLanguage | shiki.LanguageInput)[]) => Promise<void>;
    loadTheme: (...themes: (shiki.ThemeInput | "none")[]) => Promise<void>;
    setTheme: (theme: Options["theme"]) => Promise<void>;
    setLang: (lang: Options["lang"]) => Promise<void>;
}>;

export { type SimpleEditor, createEditor };
