import { CodeJar } from 'codejar';
import { createHighlighterCore, createJavaScriptRegexEngine, createWasmOnigEngine } from 'shiki';
import { withLineNumbers } from 'codejar-linenumbers';
import 'codejar-linenumbers/es/codejar-linenumbers.css';

const getTheme = (theme) => import(`../node_modules/shiki/dist/themes/${theme}.mjs`);
const getLang = (lang) => import(`../node_modules/shiki/dist/langs/${lang}.mjs`);
async function createEditor(el, options) {
  if (!options.lang || !options.theme) {
    throw new Error("lang and theme are required");
  }
  if (!options.code)
    options.code = "";
  let shiki = await createHighlighterCore({
    themes: [getTheme(options.theme)],
    langs: [getLang(options.lang)],
    engine: options.useJSEngine ? createJavaScriptRegexEngine({
      forgiving: true
    }) : createWasmOnigEngine(import('shiki/wasm'))
  });
  const setHighlight = (editor, _pos) => {
    const html = shiki.codeToHtml(editor.textContent || "", {
      lang: options.lang,
      theme: options.theme,
      transformers: [{
        pre: () => {
        }
      }]
    });
    editor.innerHTML = html;
  };
  let codeEditor = CodeJar(
    el,
    withLineNumbers(setHighlight),
    options
  );
  codeEditor.updateCode(options.code);
  return Object.assign({}, codeEditor, {
    destroy: () => {
      codeEditor.destroy();
      shiki.dispose();
      shiki = null;
      codeEditor = null;
    },
    update(options2) {
      if (options2.code)
        codeEditor.updateCode(options2.code);
      if (options2.lang)
        this.setLang(options2.lang);
      if (options2.theme)
        this.setTheme(options2.theme);
      codeEditor.updateOptions(options2);
    },
    loadLanguage: shiki.loadLanguage,
    loadTheme: shiki.loadTheme,
    setTheme: async (theme) => {
      try {
        options.theme = theme;
        await shiki.loadTheme(getTheme(theme));
        setHighlight(el);
      } catch (error) {
      }
    },
    setLang: async (lang) => {
      try {
        options.lang = lang;
        await shiki.loadLanguage(getLang(lang));
        setHighlight(el);
      } catch (error) {
      }
    }
  });
}

export { createEditor };
